/**
 * @file
 * @brief Application.hppの実装を記述する。
 * @author akino
 */
#include <XBase/Application.hpp>

//-----------------------------------------------------------
#include <XBase/Thread.hpp>
#include <XBase/Time.hpp>
#include <XBase/TimeSpan.hpp>
#include "XBaseNSApp.h"

//------------------------------------------------------------
namespace XBase {
//-----------------------------------------------------------
void Application::quit()
{
    // フラグを立てる
    mEXT.doQuit = true;
    
    // 終了処理
    XBaseNSApp_Terminate();
}
    
//-----------------------------------------------------------
AppEvent Application::receiveEventCore()
{
    // イベント処理
    XBaseNSApp_PollEvent();
     
    // 終了要求が来たら終了
    if ( mEXT.doQuit )
    {
        return AppEvent_Quit;
    }
    
    {// 60フレ同期
        s64 currentTicks = s64();
        while ( true )
        {
            currentTicks = Time::LocalTime().ticks();
            if ( ( currentTicks - mEXT.prevUpdateTicks ) < 166666 )
            {
                Thread::Sleep( TimeSpan::FromMilliseconds( 1 ) );
                continue;
            }
            break;
        }
        mEXT.prevUpdateTicks = currentTicks;
    }
    
    // 通常は更新
    return AppEvent_Update;
}
    
//------------------------------------------------------------
Application_EXT::Application_EXT()
: prevUpdateTicks( Time::LocalTime().ticks() )
, doQuit( false )
{
    XBaseNSApp_Initialize();
}

//------------------------------------------------------------
Application_EXT::~Application_EXT()
{
    XBaseNSApp_Finalize();
}
    
//------------------------------------------------------------
}
//------------------------------------------------------------
// EOF
