/**
 * @file
 * @brief Display.hppの実装を記述する。
 * @author akino
 */
#include <XBase/Display.hpp>

//------------------------------------------------------------
#include <XBase/Application.hpp>
#include <XBase/Ref.hpp>
#include "XBaseNSWindow.h"
#include <OpenGL/OpenGL.h>
//------------------------------------------------------------
namespace XBase {
//------------------------------------------------------------
uint Display::screenCount()const
{
    return 1;
}

//------------------------------------------------------------
Screen& Display::screenAtIndex( const uint aIndex )
{
    XBASE_RANGE_ASSERT_MAX( aIndex , screenCount() );
    return mainScreen();
}

//------------------------------------------------------------
Screen& Display::mainScreen()
{
    return *mEXT.mainScreen;
}

//------------------------------------------------------------
void Display::show()
{
    mEXT.isClosed = 0;    
    XBaseNSWindow_Show( mEXT.windowPtr );
}

//------------------------------------------------------------
bool Display::isClosed()const
{
    return mEXT.isClosed != 0;
}

//------------------------------------------------------------
Display_EXT::Display_EXT( const DisplayContext& aContext )
: windowPtr( 0 )
, mainScreen()
, isClosed( 0 )
{
    // Window作成
    windowPtr = XBaseNSWindow_Create(
        aContext.locationX() 
        , aContext.locationY() 
        , aContext.width() 
        , aContext.height() 
        , &isClosed 
        );
    
    // メインスクリーンの作成
    mainScreen.init( Ref( *this ) , aContext.width() , aContext.height() );
}

//------------------------------------------------------------
Display_EXT::~Display_EXT()
{    
    // メインスクリーンの削除
    mainScreen.release();
    
    // Window解放
    XBaseNSWindow* ptr = windowPtr;
    windowPtr = 0;
    XBaseNSWindow_Destroy( ptr );
}
    
//------------------------------------------------------------
}
//------------------------------------------------------------
// EOF
