/**
 * @file
 * @brief XBaseNSWindow.hの実装を記述する。
 * @author akino
 */
#include "XBaseNSWindow.h"

//-----------------------------------------------------------
#import <Cocoa/Cocoa.h>

//-----------------------------------------------------------
@interface XBaseOpenGLWindow : NSWindow
{
@private
    int* mIsClosedPtr;
}
- (int**) isClosedPtr;
@end

//-----------------------------------------------------------
@implementation XBaseOpenGLWindow
- (id) initWithContentRect: (NSRect)rect styleMask:(NSUInteger)wndStyle backing:(NSBackingStoreType)bufferingType defer:(BOOL)deferFlg
{
	[super initWithContentRect:rect styleMask:wndStyle backing:bufferingType defer:deferFlg];
            
	[[NSNotificationCenter defaultCenter] 
     addObserver:self
     selector:@selector(windowDidResize:)
     name:NSWindowDidResizeNotification
     object:self];
    
	[[NSNotificationCenter defaultCenter]
     addObserver:self
     selector:@selector(windowWillClose:)
     name:NSWindowWillCloseNotification
     object:self];
    
	[self setAcceptsMouseMovedEvents:YES];
    
	return self;
}

- (void) windowDidResize: (NSNotification *)notification
{
}

- (void) windowWillClose: (NSNotification *)notification
{
    *mIsClosedPtr = 1;
}

- (int**) isClosedPtr
{
    return &mIsClosedPtr;
}
@end
//-----------------------------------------------------------
struct XBaseNSWindow* XBaseNSWindow_Create(
    const float aPosX
    , const float aPosY
    , const float aWidth
    , const float aHeight
    , int* aIsClosedPtr 
    )
{    
    NSAutoreleasePool* pool=[[NSAutoreleasePool alloc] init];
    
	NSRect contRect;
	contRect = NSMakeRect( aPosX , aPosY , aWidth , aHeight );	
	unsigned int winStyle=
    NSTitledWindowMask|
    NSClosableWindowMask|
    NSMiniaturizableWindowMask;
	
	XBaseOpenGLWindow* window =[XBaseOpenGLWindow alloc];
    *[window isClosedPtr] = aIsClosedPtr;
    
	[window
     initWithContentRect:contRect
     styleMask:winStyle
     backing:NSBackingStoreBuffered 
		defer:NO];
    
    
    [pool release];
    
    return (struct XBaseNSWindow*)window;
}

//-----------------------------------------------------------
void XBaseNSWindow_Destroy( struct XBaseNSWindow* aWindow )
{
}

//-----------------------------------------------------------
void XBaseNSWindow_Show( struct XBaseNSWindow* aWindow )
{   
    NSAutoreleasePool* pool=[[NSAutoreleasePool alloc] init];
    
    XBaseOpenGLWindow* window = (XBaseOpenGLWindow*)aWindow;
    
    **[window isClosedPtr] = 0;
	[window makeKeyAndOrderFront:nil];
	[window makeMainWindow];
    
	[NSApp activateIgnoringOtherApps:YES];
    
    [pool release];
}

//-----------------------------------------------------------
// EOF
