﻿/**
 * @file
 * @brief Display.hppの実装を記述する。
 * @author akino
 */
#include <XBase/Display.hpp>

//------------------------------------------------------------
#include <XBase/Application.hpp>
#include <XBase/Pointer.hpp>
#include <XBase/Ref.hpp>
#include <XBase/SDKHeader.hpp>

//------------------------------------------------------------
namespace XBase {
//------------------------------------------------------------
namespace {
    Pointer< Display_EXT > tCurrentDisplay;
}

//------------------------------------------------------------
uint Display::screenCount()const
{
    return 1;
}

//------------------------------------------------------------
Screen& Display::screenAtIndex( const uint aIndex )
{
    XBASE_RANGE_ASSERT_MAX( aIndex , screenCount() );
    return mainScreen();
}

//------------------------------------------------------------
Screen& Display::mainScreen()
{
    return *mEXT.mainScreen;
}

//------------------------------------------------------------
void Display::show()
{
    mEXT.isClosed = false;
	ShowWindow( mEXT.window , SW_SHOWNORMAL );
	UpdateWindow( mEXT.window );
}

//------------------------------------------------------------
bool Display::isClosed()const
{
    return mEXT.isClosed;
}

//------------------------------------------------------------
Display_EXT::Display_EXT( const DisplayContext& aContext )
: window()
, windowClass()
, message()
, mainScreen()
, isClosed( true )
{
    // インスタンスの取得
    HINSTANCE hinstance = (HINSTANCE)GetModuleHandle(0);

    // Windowクラスのセットアップ
	windowClass.cbSize        = sizeof(windowClass);       
	windowClass.style         = CS_HREDRAW | CS_VREDRAW;
	windowClass.lpfnWndProc   = WindowProcess;
	windowClass.cbClsExtra    = 0;
	windowClass.cbWndExtra    = 0;
	windowClass.hInstance     = hinstance;
	windowClass.hIcon         = LoadIcon( NULL, IDI_APPLICATION );
	windowClass.hCursor       = LoadCursor( NULL, IDC_ARROW );
	windowClass.hbrBackground = (HBRUSH)GetStockObject( WHITE_BRUSH );
	windowClass.lpszMenuName  = NULL;
	windowClass.lpszClassName = L"Cross Framework Application";
	windowClass.hIconSm       = LoadIcon ( NULL , IDI_APPLICATION );
	RegisterClassEx( &windowClass );

    // ウィンドウの作成
	window = CreateWindow(
        L"Cross Framework Application"
        , L"Window Title"
        , WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX
        , aContext.locationX()
        , aContext.locationY()
        , aContext.width()
        , aContext.height()
        , 0 // hWndParent
        , 0 // hMenu
        , hinstance
        , 0 // lpParam
        );

    // メインスクリーンの作成
    mainScreen.init( Ref( *this ) , aContext.width() , aContext.height() );
}

//------------------------------------------------------------
void Display_EXT::pollEvent( Application& )
{
    // 今のディスプレイを設定
    tCurrentDisplay.set( *this );

    // メッセージ解析
    while ( PeekMessage( &message , window , 0 , 0 , PM_REMOVE ) != 0 )
    {
	    TranslateMessage( &message );
	    DispatchMessage( &message );
    }
    
    // 今のディスプレイを設定解除
    tCurrentDisplay.unset( *this );
}

//------------------------------------------------------------
LRESULT Display_EXT::WindowProcess( HWND aHWND , UINT aMsg , WPARAM aWParam , LPARAM aLParam )
{
    return tCurrentDisplay->windowProcess( aHWND , aMsg , aWParam , aLParam );
}

//------------------------------------------------------------
LRESULT Display_EXT::windowProcess( HWND aHWND , UINT aMsg , WPARAM aWParam , LPARAM aLParam )
{
    switch ( aMsg ) 
    {
    case WM_DESTROY:
        isClosed = true;
        PostQuitMessage(0);
        return 0;
    }

    return DefWindowProc( aHWND , aMsg , aWParam , aLParam );
}

//------------------------------------------------------------
}
//------------------------------------------------------------
// EOF
