﻿/**
 * @file
 * @brief Math型を記述する。
 * @author akino
 */
#if defined(XBASE_INCLUDED_MATH_HPP)
#else
#define XBASE_INCLUDED_MATH_HPP

//------------------------------------------------------------
#include <XBase/BuiltInTypes.hpp>

//------------------------------------------------------------
namespace XBase {
    class Angle;
}

//------------------------------------------------------------
namespace XBase {
/// @addtogroup XBase-Math
//@{
    /**
     * @brief 数学系関数を集めた空間。
     */
    //@{
    struct Math
    {
        //============================================================
        /// @name 選択
        //@{

        /// 最小値を取得する。
        template< typename T >
        static const T Min( const T aA1 , const T aA2 )
        {
            return aA1 < aA2 ? aA1 : aA2;
        }
        
        /// 最大値を取得する。
        template< typename T >
        static const T Max( const T aA1 , const T aA2 )
        {
            return aA1 < aA2 ? aA2 : aA1;
        }

        //@}

        //============================================================
        /// @name 範囲
        //@{
        /**
         * @brief クランプする。
         * @return aMin <= result <= aMax
         * @param aMin 最小値。
         * @param aMax 最大値。
         * @param aValue 対象となる値。
         */
        static const f32 ClampF32( f32 aMin , f32 aValue , f32 aMax );

        /**
         * @brief 範囲内の値か。
         * @return aMin <= aValue <= aMax ならtrue。
         */
        static bool IsInRangeF32( f32 aMin , f32 aValue , f32 aMax );
        //@}

        //============================================================
        /// @name 比較（StrictがつかないF32関数は誤差を許容する）
        //@{
        static bool IsEqualsF32( f32 value1 , f32 value2 ); ///< 値が等しいか。
        static bool IsLessEqualsF32( const f32 lhs , const f32 rhs ); ///< lhs <= rhs。
        static bool IsLessF32( const f32 lhs , const f32 rhs ); ///< lhs < rhs。
        static bool IsZeroF32( const f32 ); ///< ゼロか。
        //@}

        //============================================================
        /// @name 三角関数
        //@{
        static const f32 SinF32( const Angle& ); ///< sineを求める。
        static const f32 CosF32( const Angle& ); ///< cosineを求める。
        static const f32 TanF32( const Angle& ); ///< tangentを求める。
        static const f32 CecF32( const Angle& ); ///< cosecant(1/sin)を求める。
        static const f32 SecF32( const Angle& ); ///< secant(1/cos)を求める。
        static const f32 CotF32( const Angle& ); ///< cotangent(1/tan)を求める。
        //@}

        //============================================================
        /// @name 乗算
        //@{
        static const f32 SqrtF32( f32 val ); ///< 平方根を求める。
        //@}
    };
    //@}
//@}
}
//------------------------------------------------------------
#endif
// EOF
